var util = require('util');
var mysql = require('@incar/mysql-pool');
// 数据库对象类型化

module.exports = {
    // 使src继承自cls
    subclass: function(src, CLS){
        function __(src){
            for(var property in src) {
                if (src.hasOwnProperty(property))
                    this[property] = src[property];
            }
        }
        __.prototype = new CLS();
        return new __(src);
    },
    
    // 支持正则表达式的高级查询过滤条件
    makeWhereClause: function(filter){
        var sql = "";
        var args = [];
        if(filter) {
            for (var field in filter) {
                args.push(field);
                if(args.length> 1) sql += " and ";
                //判断过滤条件是否为正则表达式
                if (filter[field] instanceof RegExp) {
                    sql += "?? REGEXP ?";
                    // 由于JS的正则表达式toString()后前后各有一个'/',而mysql的不需要,因此只保留中间部分
                    var strX = filter[field].toString();
                    args.push(strX.substring(1, strX.lastIndexOf('/')));
                }
                else{
                    if(filter[field] === null){
                        sql += "?? is null";
                    }
                    else if(typeof filter[field] === "string"){
                        if(filter[field].substr(0, 2) === "<>"){
                            sql += "?? <> ?";
                            args.push(filter[field].substr(2));
                        }
                        else if(filter[field].substr(0, 2) === "><"){
                            sql +="?? between ? and ?";
                            var num = filter[field].substr(2).split(',');
                            args.push(num[0]);
                            args.push(num[1]);
                        }
                        else if(filter[field].substr(0, 2) === "<="){
                            sql += "?? <= ?";
                            args.push(filter[field].substr(2));
                        }
                        else if(filter[field].substr(0, 2) === ">="){
                            sql += "?? >= ?";
                            args.push(filter[field].substr(2));
                        }
                        else if(filter[field][0] === "<"){
                            sql += "?? < ?";
                            args.push(filter[field].substr(1));
                        }
                        else if(filter[field][0] === ">"){
                            sql += "?? > ?";
                            args.push(filter[field].substr(1));
                        }
                        else{
                            sql += "?? = ?";
                            args.push(filter[field]);
                        }
                    }
                    else {
                        sql += "?? = ?";
                        args.push(filter[field]);
                    }
                }
            }
        }
        return {where:sql, args: args};
    },
    
    // 组装支持分页的SQL
    assemblyPagingSQL: function(tableName, filter, sortBy, pageSize, nextMarker) {
        // select & from
        var sqlTemplate =  "SELECT * FROM(SELECT P.*,@rowNum:=@rowNum+1 AS rowNo\n"+
                            "FROM (SELECT T.*\n"+
                            "\tFROM ?? AS T\n"+
                            "\t%s%s) AS P JOIN (SELECT @rowNum := 0) R) U";
        var sqlCountTemplate = "SELECT count(*) AS rowsCount FROM ?? %s";
        var args = [tableName];
        // where
        var sqlWhere = "";
        var whereClause = this.makeWhereClause(filter);
        if(whereClause.args.length > 0) {
            sqlWhere = "WHERE " + whereClause.where;
            for (var i in whereClause.args) args.push(whereClause.args[i]);
        }
        // order by 
        var sqlOrderBy = "";
        if (sortBy) {
            sqlOrderBy = "\n\tORDER BY ??";
            var lastChar = sortBy.charAt(sortBy.length - 1);
            if (lastChar === "+" || lastChar === "-") {
                args.push(sortBy.substr(0, sortBy.length - 1));
                if (lastChar === "-") sqlOrderBy += " DESC"
            }
            else {
                args.push(sortBy);
            }
        }
        // where & order by
        var sql = util.format(sqlTemplate, sqlWhere, sqlOrderBy);
        var sqlCount = util.format(sqlCountTemplate, sqlWhere);
        // limit
        if (!isNaN(pageSize) && pageSize >= 0) {
            if(nextMarker){
                sql += "\nLIMIT ?,?";
                args.push(nextMarker-1);
                args.push(pageSize);
            }
            else{
                sql += "\nLIMIT ?";
                args.push(pageSize);
            }
        }
        else{
            // 允许pageSize传-1,这时表示返回所有余下的行数
            if(nextMarker) {
                sql += "\nWHERE U.rowNo >= ?";
                args.push(nextMarker);
            }
        }
        
        return {sql:sql, sqlCount:sqlCount, args:args};
    },
    
    // 利用表结构信息进行插入
    commonInsertG:function*(table,object){
        var sql = "SHOW COLUMNS FROM ??;";
        var fSql = " INSERT ?? SET ?";
        var columns = yield mysql.pool.queryP(sql,[table]);
        var newOrg = {};
        for(var i=0;i<columns.length;i++){
            for(var field in object){
                if(field == columns[i].Field){
                    newOrg[field] = object[field];
                }
            }
        }
        return yield mysql.pool.queryP(fSql,[table,newOrg]);
    },

    // 利用表结构信息进行更新
    commonUpdateG:function*(table,oldObject,object){
       var sql = "SHOW COLUMNS FROM ??;";
       var uSql = "UPDATE ?? SET ";
       var args=[table];
       var columns = yield mysql.pool.queryP(sql,[table]);
       var flag = false;
       for(var i=0;i<columns.length;i++) {
           if (oldObject[columns[i].Field] !== object[columns[i].Field]) {
               if (flag) uSql += ",";
               flag = true;
               uSql += "??= ?";
               args.push(columns[i].Field);
               args.push(object[columns[i].Field]);
           }
       }
       if(flag){
           uSql +=" WHERE id = ?;";
           args.push(object.id);
           yield mysql.pool.queryP(uSql,args);
       }
   }

};
